-- Used on the grappling hook. It ropes the firer and target together.
function CosXMag(Mag,Ang)
	return Mag * math.cos(Ang*math.pi/180);
end

function SinYMag(Mag,Ang)
	return Mag * math.sin(Ang*math.pi/180);
end

function DrawGrappleLine(StartPosition, EndPosition, DrawingQuality, Thing2DrawWith)  -- Draws a line
	local VectorToDestination = SceneMan:ShortestDistance(StartPosition,EndPosition,SceneMan.SceneWrapsX);
	local ParticleCount = VectorToDestination.Magnitude * DrawingQuality;
	local DistanceMultiplier = (1 / DrawingQuality);
	local DirToDestination = VectorToDestination.AbsRadAngle;
	for ParticleNumber = 0, ParticleCount, 1 do
		local Particle = CreateMOSRotating(Thing2DrawWith);
		Particle.Pos = StartPosition + VectorToDestination.Normalized * DistanceMultiplier * ParticleNumber;
		Particle.RotAngle = DirToDestination;
		MovableMan:AddParticle(Particle);
	end
end

function BetterGrapple(ObjectA, ObjectB, MaxDistance, BreakDist, Strength)
	
	local DistVector = SceneMan:ShortestDistance(ObjectA.Pos,ObjectB.Pos,SceneMan.SceneWrapsX); -- Distance between the two objects as a vector
	local Dist = DistVector.Magnitude; -- And in pixels

	local RootObjectA = nil;
	local RootObjectB = nil;

	if not MovableMan:ValidMO(ObjectA) then
		RootObjectA = MovableMan:GetMOFromID(ObjectA.RootID);
	else
		RootObjectA = ObjectA;
	end


	if not MovableMan:ValidMO(ObjectB) then
		RootObjectB = MovableMan:GetMOFromID(ObjectB.RootID);

	else
		RootObjectB = ObjectB;
	end

	local StretchDist = DistVector - Vector(MaxDistance * math.cos(DistVector.AbsRadAngle), -1 * MaxDistance * math.sin(DistVector.AbsRadAngle)); -- Things that have to do with Hooke's law and springs
	local NetMass = RootObjectA.Mass + RootObjectB.Mass;
	local mag = StretchDist.Magnitude;
	local ForceA = mag * Strength * (RootObjectA.Mass)/NetMass;
	local ForceB = mag * Strength * (RootObjectB.Mass)/NetMass;

	local ForceVectorA = Vector(math.cos(DistVector.AbsRadAngle) * ForceA, math.sin(DistVector.AbsRadAngle) * ForceA * -1);
	local ForceVectorB = Vector(math.cos(DistVector.AbsRadAngle) * ForceB, math.sin(DistVector.AbsRadAngle) * ForceB * -1);

	if DistVector.X < 0 then -- If ObjectB is to the left of ObjectA, flip the force direction
		ForceVectorB:FlipX(true);
		ForceVectorB:FlipY(true);
	else
		ForceVectorB:FlipX(true);
		ForceVectorB:FlipY(true);
	end

	if Dist > MaxDistance then -- if they the two objects are too far apart, pull them together.
		ObjectA:AddImpulseForce(ForceVectorA, Vector(0,0));
		ObjectB:AddImpulseForce(ForceVectorB, Vector(0,0));
	end

	DrawGrappleLine(ObjectA.Pos, ObjectB.Pos, 0.18, "Line Grapple Gun"); -- Draw the grappling cord

	if Dist > BreakDist then -- If the rope is too long, break it
		return true;
	else
		return false;
	end	
end


function Create(self)
	self.Grab = 0; -- It hasn't grabbed anything yet
	self.LinePow = 15; -- Pull streangth modifier

	self.Parent = nil;

	local curdist = 60; -- Gets the actor shot the hook and sets it to self.Parent
	for actor in MovableMan.Actors do
	local dist = SceneMan:ShortestDistance(self.Pos,actor.Pos,true).Magnitude;
		if dist < curdist then
			curdist = dist;
			self.Parent = actor;
		end
	end

	if self.Parent.RootID == nil then -- If there is no parent, delete
		MovableMan:RemoveMO(self);
	end

	self.MaxDistAtm = 250; -- Max distance before the two objects get pulled together
	self.Broken = false; -- The rope has not been broken yet
end


function Update(self)

	self.ToSettle = false;

	if MovableMan:ValidMO(MovableMan:GetMOFromID(self.Parent.RootID)) then -- If the parent exists do the following
		if self.Target ~= nil then
			if not MovableMan:ValidMO(MovableMan:GetMOFromID(self.Target.RootID)) then
				MovableMan:RemoveMO(self);
			end
		end
		if (self.Grab == 0) then -- If we haven't grabbed anything yet
			DrawGrappleLine(self.Parent.Pos, self.Pos, 0.18, "Line Grapple Gun"); -- Draw the grappling cord
			self.Frame = 0;
			local Dist = SceneMan:ShortestDistance(self.Parent.Pos,self.Pos,true).Magnitude;
			if Dist > (self.MaxDistAtm + 35) then
				MovableMan:RemoveMO(self);
			end
			i = Vector(0,0);
			j = SceneMan:CastMORay(self.Pos, self.Vel.Normalized * 20, 0, 0, false, 5);
			if j ~= 255 then -- If the hook is about to hit a MOID, set it as the target
				self.Target = MovableMan:GetMOFromID(j);
				self.Grab = 1
				self.Frame = 1;
				self.PinStrength = 5000; -- Pins the hook, for some reason pinned MOSRots don't settle
				self.Pos = self.Target.Pos;

				local clicksound = CreateAEmitter("Click Sound Emitter"); -- Play a "click" sound
				clicksound.Pos = self.Target.Pos;
				MovableMan:AddParticle(clicksound);

				local Impale = CreateMOPixel("Particle Impale"); -- Spawn a sharp MOPixel
				Impale.Pos = self.Pos;
				Impale.Vel = self.Vel;
				MovableMan:AddParticle(Impale);
			elseif SceneMan:CastStrengthRay(self.Pos, self.Vel.Normalized * 20, 0, i, 0, 0, true) then --If the hook is about to hit terrain, pin it
				self.Pos = i;
				self.PinStrength = 5000; -- Pins the hook, for some reason pinned MOSRots don't settle
				self.Grab = 2;
				self.Frame = 1;

				local clicksound = CreateAEmitter("Click Sound Emitter"); -- Play a "click" sound
				clicksound.Pos = self.Pos;
				MovableMan:AddParticle(clicksound);
			end
		elseif (self.Grab == 1)  then -- If we've grabbed a MOID then
			if MovableMan:ValidMO(MovableMan:GetMOFromID(self.Target.RootID)) then -- If the target still exists rope it together with the parent
				self.Pos = self.Target.Pos;
				if self.Broken == false then
					self.Broken = BetterGrapple(self.Parent, self.Target, self.MaxDistAtm, self.MaxDistAtm + 50, self.LinePow);
				end
			else
	        		MovableMan:RemoveMO(self); -- If it doesn't, delete
			end
		elseif (self.Grab == 2) then -- if we've grabbed terrain
			if MovableMan:ValidMO(MovableMan:GetMOFromID(self.Parent.RootID)) then -- If the parent still exists rope it together with the hook and pin the hook to the terrain
				self.Vel = Vector(0,0);
				if self.Broken == false then
					self.Broken = BetterGrapple(self.Parent, self, self.MaxDistAtm, self.MaxDistAtm + 50, self.LinePow);
				end
			else
	        		MovableMan:RemoveMO(self); -- Otherwise, delete
			end
		end

		if self.Parent.Team ~= nil then
			if (ActivityMan:GetActivity():GetControlledActor(self.Parent.Team).RootID == self.Parent.RootID) then -- if the parent is player controlled
				if UInputMan:KeyHeld(75) then -- If the spacebar is held, break the rope
					MovableMan:RemoveMO(self);
				end

				if UInputMan:KeyHeld(62) or UInputMan:KeyHeld(89) then
					if self.MaxDistAtm < 500 then -- If '+' is held, lengthen the rope
						self.MaxDistAtm = self.MaxDistAtm + 5;
					end
				end

				if UInputMan:KeyHeld(61) or UInputMan:KeyHeld(88) then
					if self.MaxDistAtm > 0 then -- If '-' is held, shorten the rope
						self.MaxDistAtm = self.MaxDistAtm - 5;
					end
				end
			end
		end
  	else
        	MovableMan:RemoveMO(self); -- If the parent doesn't exist, delete
  	end


	if self.Broken then -- if the rope is broken, delete
		MovableMan:RemoveMO(self);
	end


	if self.Target ~= nil then -- if either the target or the parent settles, delete
		if self.Parent.ToSettle or self.Target.ToSettle then
			MovableMan:RemoveMO(self);
		end
	elseif self.Parent.ToSettle then
		MovableMan:RemoveMO(self);
	end
end
